
package agent.awtgrep;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Serializable;
import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Vector;

import java.awt.*;

import agentspace.Context;
import agentspace.Agent;
import agentspace.InvalidURLException;
import agentspace.Mobile;
import agentspace.event.AgentEvent;

public class AWTGrep implements Agent, Mobile {
	GrepWindow window = null;
	URL homeURL = null;
	String pattern = null;
	String path = null;
	URL targetURL = null;
	String result = "";
	boolean searching = false;
	
	public static void main(String[] args) {
		AWTGrep grep = new AWTGrep();
		grep.create(null, null);
	}
		
	class GrepWindow extends Frame {
		TextArea area;
		Context context;
		ActionListener listener;
		Panel grepPanel;
		Panel networkPanel;
		TextField patternField;
		TextField pathField;
		TextField addrField;
		Button button;
		
		public GrepWindow(Context context) {
			super("Mobile Grep Agent");
			this.context = context;
			listener = new GrepAction();
			grepPanel = new Panel();
			add(new Label("Pattern:"));
			patternField = new TextField("public", 12);
			grepPanel.add(patternField);
			pathField = new TextField("agent/awtgrep/AWTGrep.java", 24);
			add(new Label("Target file(s):"));
			grepPanel.add(pathField);

			networkPanel = new Panel();
			add(new Label("URL"));
			addrField = new TextField(agentspace.AgentServer.getTargetURL().toString(), 23);
			networkPanel.add(addrField);

			button = new Button("Search");
			button.setActionCommand("Search");
			button.addActionListener(listener);
			networkPanel.add(button);
			
			area = new TextArea(80, 20);
			
			setLayout(new BorderLayout());
			add(grepPanel, BorderLayout.NORTH);
			add(area, BorderLayout.CENTER);
			add(networkPanel, BorderLayout.SOUTH);
			setSize(500, 300);
		}
		
		public void appendText(String result) {
			String content = area.getText();
			area.setText(result+"\n"+content);
		}
		
		public void setContext(Context context) {
			this.context = context;
		}

		class GrepAction implements ActionListener, Serializable {
			public void actionPerformed(ActionEvent evt) {
				String action = evt.getActionCommand();
				if (action.equals("Search")) {
					try {
						pattern = patternField.getText();
						path = pathField.getText();
						targetURL = new URL(addrField.getText());
//						System.out.println("Grep: targetURL = "+targetURL);
						context.move(targetURL);
					} 
					catch (MalformedURLException e) {
						e.printStackTrace();
					} 
					catch (IOException e) {
						e.printStackTrace();
					} 
					catch (InvalidURLException e) {
						e.printStackTrace();
					}
				}
			}
		}
	}
	
	public AWTGrep() {
		System.out.println("Grep: an instance created.");
	}
	/* 
	 * @see agentspace.SerialiableAgent#create(agentspace.event.AgentEvent, agentspace.Context)
	 */
	public void create(AgentEvent evt, Context context) {
		System.out.println("Grep: method create() called");
		if (homeURL == null) {
			homeURL = context.getHostURL();
		}
		window = new GrepWindow(context);
		window.setVisible(true);
	}
	/* 
	 * @see agentspace.SerialiableAgent#destroy(agentspace.event.AgentEvent, agentspace.Context)
	 */
	public void destroy(AgentEvent evt, Context context) {
		System.out.println("Grep: method destroy() called");
		window.setVisible(false);
	}
	/* 
	 * @see agentspace.SerialiableAgent#suspend(agentspace.event.AgentEvent, agentspace.Context)
	 */
	public void suspend(AgentEvent evt, Context context) {
		System.out.println("Grep: method suspend() called");
		window.setVisible(false);
	}

	/* 
	 * @see agentspace.SerialiableAgent#resume(agentspace.event.AgentEvent, agentspace.Context)
	 */
	public void resume(AgentEvent evt, Context context) {
		System.out.println("Grep: method resume() called");
		window.setVisible(true);
	}
	/* 
	 * @see agentspace.MobileAgent#arrive(agentspace.event.AgentEvent, agentspace.Context)
	 */
	public void arrive(AgentEvent evt, Context context) {
//		System.out.println("Grep: method arrive() called");
//		System.out.println("Grep: home = "+homeURL);
//		System.out.println("Grep: current = "+context.getHostURL());
		InetAddress	homeAddr;
		try {
			homeAddr = InetAddress.getByName(homeURL.getHost());
			InetAddress	currentAddr = InetAddress.getByName(context.getHostURL().getHost()); 
			if (homeAddr.equals(currentAddr) && 
					homeURL.getPort() == context.getHostURL().getPort() && searching ) { 
				searching = false;
//				System.out.println("Grep: arriving at the home");
				window.appendText(result);
				window.setContext(context);
				window.setVisible(true);
			}
			else {
//				System.out.println("Grep: arriving at remote host");
				result = compare(path, pattern);
				searching = true;
				context.move(homeURL);
			}
		}
		catch (Exception ex) {
			ex.printStackTrace();
		}
	}
		
	/* 
	 * @see agentspace.MobileAgent#leave(agentspace.event.AgentEvent, agentspace.Context)
	 */
	public void leave(AgentEvent evt, Context context) {
		System.out.println("Grep: method leave() called");
		window.setVisible(false);
	}

	public static String compare(String fname, String pattern) {
		Vector list = new Vector();
		BufferedReader reader = null;
		try {
			FileInputStream in = new FileInputStream(fname);
			reader = new BufferedReader(new InputStreamReader(in));
		}
		catch (IOException e) {
			System.err.println(e);
		}
		String line = null;
		try {
			while((line = reader.readLine()) != null) {
				if (line.indexOf(pattern) >= 0) {
					System.out.println("Grep("+pattern+"): line = "+line);
					list.addElement(line);
				}
			}
		}
		catch (IOException e) {
			System.err.println(e);
		}
		String result = "";
		for (int i = 0 ; i < list.size() ; i++ ) {
			result = result + "\n" + (String)list.elementAt(i);
		}
		return result;
	}
}
