/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/13
 */

package agentspace.monitor.table;

import java.awt.BorderLayout;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
//import javax.swing.table.DefaultTableModel;
//import javax.swing.table.TableModel;

import agentspace.runtime.AgentContainer;

public class AgentTableView extends JPanel implements FocusListener, MouseListener, KeyListener {
	JTable tableView;
	AgentTableModel tableModel = null;
	AgentTableToolBar toolbar = null;
	
	public AgentTableView(AgentTableControl listener, AgentTableToolBar toolbar) {
		setLayout(new BorderLayout());
		this.toolbar = toolbar;

		tableModel = new AgentTableModel();
		((AgentTableControl) listener).setTableModel(tableModel);
/*
 *  when sorting items is needed, please replace the following commented codes. 
		AgentTableSorter sorter = new AgentTableSorter(tableModel);
		tableView = new JTable(sorter);
		sorter.addMouseListenerToHeaderInTable(tableView);
*/
		tableView = new JTable(tableModel);
		
		tableModel.setTable(tableView);
		tableModel.addTableModelListener(listener);
		tableView.setSize(600, 150);
		tableView.addMouseListener(this);
		tableView.addKeyListener(this);

		JScrollPane scrollpane = new JScrollPane(tableView);
		add(scrollpane);
		setVisible(true);
	}

	public int count() {
		return tableModel.getRowCount();
	}

	synchronized public void addElement(AgentContainer ac) {
		tableModel.addElement(ac);
		updateToolBar(tableView.getSelectedRow());
	}

	synchronized public void removeElement(AgentContainer ac) {
		updateToolBar(tableView.getSelectedRow());
		tableModel.removeElement(ac);
	}

	synchronized public void changeElement(AgentContainer ac) {
		tableModel.changeElement(ac);
		updateToolBar(tableView.getSelectedRow());
	}

	synchronized public void removeElementAt(int row) {
		tableModel.removeElementAt(row);
	}

	protected void updateToolBar(int pos) {
		if (pos < 0 || count() < pos) {
			if (toolbar != null) {
				toolbar.loadButton.setEnabled(true);
				toolbar.storeButton.setEnabled(false);
				toolbar.destroyButton.setEnabled(false);
				toolbar.copyButton.setEnabled(false);
				toolbar.moveButton.setEnabled(false);
				toolbar.infoButton.setEnabled(false);
				toolbar.haltButton.setEnabled(false);
			}
		}
		else {
			AgentContainer ac = tableModel.getElementAt(pos);
			if (ac != null && toolbar != null) {
				if (ac.isMobile()) {
					toolbar.moveButton.setEnabled(true);
				} else {
					toolbar.moveButton.setEnabled(false);
				}
				if (ac.isPreservable()) {
					toolbar.storeButton.setEnabled(true);
				} else {
					toolbar.storeButton.setEnabled(false);
				}
				if (ac.isDuplicatable()) {
					toolbar.copyButton.setEnabled(true);
				} else {
					toolbar.copyButton.setEnabled(false);
				}
				if (ac.isTerminable()) {
					toolbar.destroyButton.setEnabled(true);
				} else {
					toolbar.destroyButton.setEnabled(false);
				}
				toolbar.infoButton.setEnabled(true);
				toolbar.haltButton.setEnabled(true);
			}
		}
	}

	public void mouseClicked(MouseEvent evt) {
		JTable table = (JTable)evt.getComponent();
		if (table == tableView) {
			updateToolBar(table.getSelectedRow());
		}
	}
	public void mousePressed(MouseEvent evt) {}
	public void mouseReleased(MouseEvent evt) {}
	public void mouseEntered(MouseEvent evt) {}
	public void mouseExited(MouseEvent evt) {}
	public void focusGained(FocusEvent evt) {}
	public void focusLost(FocusEvent evt) {}
	public void keyTyped(KeyEvent evt) {}
	public void keyPressed(KeyEvent evt) {
		updateToolBar(tableView.getSelectedRow());		
	}
	public void keyReleased(KeyEvent evt) {}
}