/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/7
 */

package agentspace.network.protocol;

import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Hashtable;

public class Request {
    protected String requestMethod   = null;
    protected String requestURI     = null;
    protected String requestProtocol = null;
    protected String requestVersion  = null;
    protected byte[] requestContent  = null;
    protected Hashtable attributes = null;
    
    public Request() {}

    public String getMethod() {
        return requestMethod;
      }

      public String getVersion() {
        return requestVersion;
      }

      public byte[] getContent() {
        return requestContent;
      }

      public String getURI() {
        return requestURI;
      }

      public String getProtocol() {
        return requestProtocol;
      }
      
      public Hashtable getAttributes() {
          return attributes;
      }
      
      public String getAttribute(String name) {
          if (attributes == null) {
              return null;
          }
          return (String)attributes.get(name.toUpperCase());
      }

      public String toString() {
      	return "protocol:"+requestProtocol+", method:"+requestMethod+", URI:"+requestURI;
      }
      
      public byte[] toByteArray() {
        byte[] data = null;
    	try {
    		ByteArrayOutputStream baos = new ByteArrayOutputStream(); 
    		DataOutputStream dos = new DataOutputStream(baos);
    		
    		String headerLine = "";
		    if (requestMethod != null) {
		    	headerLine = headerLine + requestMethod;
		    }
		    if (requestURI != null) {
		    	headerLine = headerLine + " " + requestURI;
		    }
		    if (requestProtocol != null && requestVersion != null) {  
		    	headerLine = headerLine + " " + requestProtocol + "/" + requestVersion + "\r\n";
    		}
    		dos.writeBytes(headerLine);
    		
    		if (requestContent != null) {
    			String value = getAttribute("Content-Length");
    			if (value == null) {
    				if (attributes == null) {
    					attributes = new Hashtable();
    				}
    				attributes.put("Content-Length", String.valueOf(requestContent.length));
    			}
    		}
    		
    		String attributeLine = "";
    		if (attributes != null && !attributes.isEmpty()) {
    			java.util.Enumeration e = attributes.keys();
    			while(e.hasMoreElements()) {
    				String attributeLines = "";
    				String attributeName = (String)e.nextElement();
    				String attributeValue = (String)attributes.get(attributeName); 
    				attributeLine = attributeName + ": " +  attributeValue + "\r\n";
    				dos.writeBytes(attributeLine);
    			}
    			if (headerLine.length() != 0 && attributeLine.length() != 0) { 
    				dos.writeBytes("\r\n");		    	
    			}
    		}
    		if (requestContent != null) {
    			dos.write(requestContent, 0, requestContent.length);
    		}
    		dos.close();
    		data = baos.toByteArray();
    	}
    	catch (IOException ex) {
    		ex.printStackTrace();
    	}
    	return data;
    }

}
