/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/7
 */

package agentspace.network.protocol.http;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.net.URL;
import java.net.URLConnection;

import agentspace.network.protocol.Request;
import agentspace.network.protocol.RequestFactory;
import agentspace.network.protocol.Response;
import agentspace.network.protocol.ResponseParser;

/**
 * @author ichiro
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class HttpURLConnection extends URLConnection {
	public final static int DEFAULT_PORT = 7000;
	private Socket connection = null;
	
	/**
	 * @param url
	 */
	protected HttpURLConnection(URL url) {
		super(url);
	}

	public synchronized InputStream getInputStream() throws IOException {
		if (!connected) {
			this.connect();
		}
		InputStream in = this.connection.getInputStream();
		return in;
	}

	public synchronized OutputStream getOutputStream() throws IOException {
		if (!connected) {
			this.connect();
		}
		OutputStream out = this.connection.getOutputStream();
		return out;
	}
	
	/* (non-Javadoc)
	 * @see java.net.URLConnection#connect()
	 */
	public void connect() throws IOException {
		try {
			if (!connected) {
				int port = url.getPort();
				if (port < 1 || port > 65535) {
					port = DEFAULT_PORT;
				}
				connection = new Socket(url.getHost(), port);
			}
			OutputStream out = this.connection.getOutputStream();
			InputStream in = this.connection.getInputStream();
			Request request = new RequestFactory();
			byte[] data = request.toByteArray();
			out.write(data);
			out.flush();
			Response response = new ResponseParser(in);
			this.connected = true;
		}
		catch (IOException ex) {
			ex.printStackTrace();
		}
	}
}
