/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/25
 */

package agentspace.network.protocol.http;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.net.URLConnection;


import agentspace.network.protocol.RequestFactory;
import agentspace.network.protocol.Response;
import agentspace.network.protocol.ResponseParser;

public class URLSessionHandler implements agentspace.network.URLSessionHandler {

	/* (non-Javadoc)
	 * @see agentspace.network.URLSessionHandler#handle(java.lang.String, byte[])
	 */
	public byte[] handle(String method, URL url, byte[] data) throws IOException {
		byte[] result = null;
		if (url == null) {
			throw new IOException("Illegal URL = "+url);
		}
		try {
			URLConnection connection = url.openConnection();
			OutputStream out = connection.getOutputStream();
			InputStream in = connection.getInputStream();
			RequestFactory request = new RequestFactory();
			request.setProtocol("HTTP");
			request.setVersion("1.0");
			if (method.equals("GET")) {
				request.setMethod("GET");
				request.setURI(url.getFile());
				out.write(request.toByteArray());
				out.flush();
				Response response = new ResponseParser(in);
				System.err.println("matp.URLSessionHandler: Response = "+response);
				if (response.getCode() < 200 || response.getCode() >= 300) {
					throw new IOException("Receiving error code "+response.getCode()+
							" ("+response.getComment()+") from server");
				}
				result = response.getContent();
			}
			else if (method.equals("HEAD")){
				request.setMethod("HEAD");
				request.setURI("/");
				out.write(request.toByteArray());
				out.flush();
				Response response = new ResponseParser(in);
				if (response.getCode() < 200 || response.getCode() >= 300) { 
					throw new IOException("Receiving error code "+response.getCode()+
							" ("+response.getComment()+") from server");
				}
			}
			else if (method.equals("PUT")){
				throw new IOException("Invalid Method");
			}
		}
		catch (IOException ex) {
			ex.printStackTrace();
		}
		return result;
	}
}
