/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/7
 */

package agentspace.network.protocol.matp;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.net.URL;
import java.net.URLConnection;

public class MatpURLConnection extends URLConnection {
	public final static int DEFAULT_PORT = 7000;
	private Socket connection = null;
	
	/**
	 * @param arg0
	 */
	protected MatpURLConnection(URL url) {
		super(url);
	}

	public synchronized InputStream getInputStream() throws IOException {
		if (!connected) {
			this.connect();
		}
		InputStream in = this.connection.getInputStream();
		return in;
	}

	public synchronized OutputStream getOutputStream() throws IOException {
		if (!connected) {
			this.connect();
		}
		OutputStream out = this.connection.getOutputStream();
		return out;
	}
	
	/* (non-Javadoc)
	 * @see java.net.URLConnection#connect()
	 */
	public void connect() throws IOException {
		
		if (!connected) {
			int port = url.getPort();
			if (port < 1 || port > 65535) {
				port = DEFAULT_PORT;
			}
//			System.out.println("MatpURLConnection: host = "+url.getHost()+", port = "+port);
			connection = new Socket(url.getHost(), port);
		}
		this.connected = true;
	}
}
