/*
 * Copyright (c) 2005 Ichiro Satoh
 * All Rights Reserved.
 * written by Ichiro Satoh
 *
 * This program was developed by Ichiro Satoh at National Institute 
 * of Informatics as a part of a mobile object system, titled 
 * "AgentSpace".  All the programs of the system is developed by 
 * Ichiro Satoh and all rights are reserved by Ichiro Satoh. 
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the
 * copyright notice and this permission notice appear in all copies of
 * the software, derivative works or modified versions, and any
 * portions thereof, and that both notices appear in supporting 
 * documentation.
 */

/**
 * @author Ichiro Satoh
 * Created on 2005/05/9
 */
package agentspace.runtime.event;

import agentspace.runtime.AgentContainer;
import agentspace.runtime.AgentStatus;

public class AgentStatusEvent {
	AgentContainer ac;
	int previous;
	int current = 0;
		
	public AgentStatusEvent(AgentContainer ac, int previous, int mask) {
		this.ac = ac;
		this.previous = (previous & ~mask) | previous;
		if (ac != null) {
			current = ac.getStatus();
		}
	}

	public AgentStatusEvent(AgentContainer ac, int previous) {
		this(ac, previous, -1);
	}

	public AgentContainer getAgentContainer() {
		return ac;
	}
	
	public int getPreviousStatus() {
		return previous;
	}

	public int getCurrentStatus() {
		return current;
	}
	
	public int getPreviousLifecycleStatus() {
		return previous & AgentStatus.LIFE_MASK;
	}

	public int getCurrentLifecybleStatus() {
		return current & AgentStatus.LIFE_MASK;
	}
	
	public int getPreviousProcessStatus() {
		return previous & AgentStatus.PROCESS_MASK;
	}

	public int getCurrentProcessStatus() {
		return current & AgentStatus.PROCESS_MASK;
	}
	
	public String toString() {
		return "AgentStatusEvent: from ("+AgentStatus.getLifeName(previous) + ", " + 
		AgentStatus.getProcessName(previous) + ") to ("+
		AgentStatus.getLifeName(current) + ", " + AgentStatus.getProcessName(current) + 
		") in Agent "+ac.getIdentifier();
	}
}
